/*
 * nokia1661_lcd_driver.c
 *
 *  Created on: Apr 17, 2019
 *      Author: Tarahi_2
 */


/*
 * nokia1616/1661 TFT lcd Driver for AVR microcontroller
 * Copyright (C) 2018 mohammad mazarei (mohammad.mazarei [at] gmail.com)
 * web site: https://sisoog.com
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of  MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "spfd54124b.h"
#include "nokia1661_lcd_driver.h"
#include "stm32f1xx_hal.h"

struct N1616Data	_lcd_data;

#define SBI(port,bit) 	port->BSRR = bit
#define CBI(port,bit) 	port->BRR = bit

/*---------------------------------------------------------------------------------------------------------------
* lcd pin functions
*/
#define LCD_PIN_FUNC(a, b) \
	void _ ## a ## _set() { SBI(LCD_PORT,(LCD_ ## b)); } \
	void _ ## a ## _clr() { CBI(LCD_PORT,(LCD_ ## b)); }

LCD_PIN_FUNC(rst, RST)
LCD_PIN_FUNC(cs, CS)
LCD_PIN_FUNC(sda, SDA)
LCD_PIN_FUNC(clk, CLK)


/*----------------------------------------------------------------------------------------------------------------
* hardware functions
*/
void _delay_ms(unsigned int ms)
{

   for(int i=0;i<10000*ms;i++);


}

static unsigned int ShiftBit[] = {0x100,0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};

void _nlcdSend(unsigned int data)
{

    _cs_clr();

        if(data & ShiftBit[0]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();
        if(data & ShiftBit[1]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();
        if(data & ShiftBit[2]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();
        if(data & ShiftBit[3]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();
        if(data & ShiftBit[4]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();
        if(data & ShiftBit[5]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();
        if(data & ShiftBit[6]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();
        if(data & ShiftBit[7]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();
        if(data & ShiftBit[8]) _sda_set(); else _sda_clr();_clk_set();_clk_clr();

    _cs_set();
}

/*----------------------------------------------------------------------------------------------------------------
* private functions
*/

void _nlcdSendCmd(unsigned char data)  { _nlcdSend(SPFD54124B_SEND_CMD | data); }
void _nlcdSendData(unsigned char data) { _nlcdSend(SPFD54124B_SEND_DATA | data); }

void _nlcdCmd1616(unsigned char cmd, unsigned int a, unsigned int b)
{
	_nlcdSendCmd(cmd);
	_nlcdSendData(a>>8);
	_nlcdSendData(a);
	_nlcdSendData(b>>8);
	_nlcdSendData(b);
}

void _nlcdSetWindow(unsigned char x, unsigned char y, unsigned char w, unsigned char h)
{
//	x += LCD_X_OFFSET;
//	y += LCD_Y_OFFSET;
/*#ifdef LCD_ROTATE90
	_nlcdCmd1616(SPFD54124B_CMD_CASET, y, y+h-1);	// column start/end
	_nlcdCmd1616(SPFD54124B_CMD_RASET, LCD_MAX_X-x-w, LCD_MAX_X-x-1);	// page start/end
#else*/
	_nlcdCmd1616(SPFD54124B_CMD_CASET, x, x+w-1);	// column start/end
	_nlcdCmd1616(SPFD54124B_CMD_RASET, y, y+h-1);	// page start/end
//#endif
	_nlcdSendCmd(SPFD54124B_CMD_RAMWR);				// RAMWR
}

void _nlcdSendPixel(rgb_color16bit color)
{
	_nlcdSendData(color >> 8);
	_nlcdSendData(color & 0xFF);
}

void _nlcdSenPixels(rgb_color16bit color, unsigned int n)
{
	while(n--) _nlcdSendPixel(color);
}

/*----------------------------------------------------------------------------------------------------------------
* public functions
*/
const unsigned int _lcd_init_list[] =
{
	SPFD54124B_CMD_SLPOUT,
	SPFD54124B_CMD_COLMOD, SPFD54124B_SEND_DATA | SPFD54124B_CMD_COLMOD_MCU16bit,
	SPFD54124B_CMD_DISPON,
	SPFD54124B_CMD_INVOFF,
	SPFD54124B_CMD_NORON
};

void nlcdInit()
{


  
  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __GPIOB_CLK_ENABLE();

  /*Configure GPIO pins : PB12 PB13 PB14 PB15 */
  GPIO_InitStruct.Pin = LCD_RST | LCD_CS | LCD_SDA | LCD_CLK;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
  
  LCD_PORT->BSRR = (1 << LCD_RST) | (1 << LCD_CS) | (1 << LCD_SDA); 
  


	_rst_clr();
	_delay_ms(500);
	_rst_set();
	_delay_ms(500);

    const unsigned int *list = &_lcd_init_list[0];
    unsigned char size = sizeof(_lcd_init_list)/sizeof(_lcd_init_list[0]);
    while(size--) _nlcdSend(*list++);
}

void nlcdClear()
{
	_nlcdSetWindow(0, 0, nlcdGetWidth(), nlcdGetHeight());
	_nlcdSenPixels(_lcd_data.bg_color, nlcdGetWidth() * nlcdGetHeight());
	_nlcdSendCmd(SPFD54124B_CMD_NOP);
	_lcd_data.charx = 0;
	_lcd_data.chary = 0;
}

void nlcdPixel(unsigned char x, unsigned char y,rgb_color16bit color)
{
	_nlcdCmd1616(SPFD54124B_CMD_CASET, x, x+1);	// column start/end
	_nlcdCmd1616(SPFD54124B_CMD_RASET, y, y+1);	// page start/end
	_nlcdSendCmd(SPFD54124B_CMD_RAMWR);			// RAMWR
	_nlcdSendData(color >> 8);
	_nlcdSendData(color & 0xFF);
}
void nlcdSetBackgroundColor(rgb_color16bit color)
{
	_lcd_data.bg_color = color;
}

void nlcdGotoCharXY(unsigned char x, unsigned char y)
{
	_lcd_data.charx = (_lcd_data.font_desc.width + _lcd_data.font_desc.h_spacing) * x;
	_lcd_data.chary = (_lcd_data.font_desc.height + _lcd_data.font_desc.v_spacing) * y;
}

void nlcdCharXY(unsigned char x, unsigned char y, rgb_color16bit color, char c)
{
	nlcdGotoCharXY(x, y);
	nlcdChar(color, c);
}

void nlcdChar(rgb_color16bit color, char c)
{



	_nlcdSetWindow(_lcd_data.charx, _lcd_data.chary, _lcd_data.charw, _lcd_data.charh);

	// shift char x
	_lcd_data.charx += _lcd_data.charw;
	if(_lcd_data.charx + _lcd_data.charw >= nlcdGetWidth()) {
		// zero char x
		_lcd_data.charx = 0;
		// shift char y
		_lcd_data.chary += _lcd_data.charh;
		if(_lcd_data.chary + _lcd_data.charh >= nlcdGetHeight()) {
			_lcd_data.charx = 0;
			_lcd_data.chary = 0;
		}
	}

/*#ifdef LCD_ROTATE90
	_nlcdSenPixels(_lcd_data.bg_color, (_lcd_data.font_desc.height+_lcd_data.font_desc.v_spacing)*_lcd_data.font_desc.h_spacing);
	for(uint8_t i=_lcd_data.font_desc.width; i--;) {
		uint8_t data = pgm_read_byte(&(_lcd_data.font_data[((uint8_t)c - 32)*_lcd_data.font_desc.width + i]));
		for(uint8_t j=0; j<8; j++) {
			if( data & (1 << j) )
				_nlcdSendPixel(color);
			else
				_nlcdSendPixel(_lcd_data.bg_color);
		}
	}
#else*/
	for(unsigned char j=0; j<_lcd_data.font_desc.height+_lcd_data.font_desc.v_spacing; j++)
	{
		for(unsigned char i=0; i<_lcd_data.font_desc.width; i++)
		{
			unsigned char data = *(&(_lcd_data.font_data[((unsigned char)c - 32)*_lcd_data.font_desc.width + i]));
			if( data & (1 << j) )
				_nlcdSendPixel(color);
			else
				_nlcdSendPixel(_lcd_data.bg_color);
		}
		_nlcdSendPixel(_lcd_data.bg_color);
	}
}

void nlcdSetFont(const unsigned char* font)
{
	memcpy(&_lcd_data.font_desc, font, sizeof(_lcd_data.font_desc));
	_lcd_data.font_data = font + _lcd_data.font_desc.header_offset;
	_lcd_data.charw = _lcd_data.font_desc.width + _lcd_data.font_desc.h_spacing;
	_lcd_data.charh = _lcd_data.font_desc.height + _lcd_data.font_desc.v_spacing;
}

char nlcdFontLastChar()
{
	return _lcd_data.font_desc.char_offset + _lcd_data.font_desc.number;
}

char nlcdFontFirstChar()
{
	return _lcd_data.font_desc.char_offset;
}

void nlcdString(rgb_color16bit color, const char* str)
{
	while(*str) nlcdChar(color, *str++);
}

void nlcdStringXY(char x, char y, rgb_color16bit color, const char* str)
{
	nlcdGotoCharXY(x, y);
	nlcdString(color, str);
}

void nlcdStringP(rgb_color16bit color, const char* str)
{
	char c = *(str);
	while(c) {
		nlcdChar(color, c);
		c = *(++str);
	}
}

void nlcdStringXYP(char x, char y, rgb_color16bit color, const char* str)
{
	nlcdGotoCharXY(x, y);
	nlcdStringP(color, str);
}

void nlcdSetOrientation(unsigned char or)
{
	unsigned char data = 0;
	_lcd_data.orientation = or;

	switch(or)
	{
		case LCD_ORIENTATION_90:
			data = SPFD54124B_CMD_MADCTR_MV | SPFD54124B_CMD_MADCTR_MX;
			break;
		case LCD_ORIENTATION_180:
			data = SPFD54124B_CMD_MADCTR_MY | SPFD54124B_CMD_MADCTR_MX;
			break;
		case LCD_ORIENTATION_270:
			data = SPFD54124B_CMD_MADCTR_MV | SPFD54124B_CMD_MADCTR_MY;
			break;
	}
	_nlcdSendCmd(SPFD54124B_CMD_MADCTR);
	_nlcdSendData(data);
}

unsigned char nlcdGetWidth()
{
	switch(_lcd_data.orientation)
	{
		case LCD_ORIENTATION_NORMAL:
		case LCD_ORIENTATION_180:
			return LCD_CTRL_WIDTH;
	}
	return LCD_CTRL_HEIGHT;
}

unsigned char nlcdGetHeight()
{
	switch(_lcd_data.orientation)
	{
		case LCD_ORIENTATION_NORMAL:
		case LCD_ORIENTATION_180:
		return LCD_CTRL_HEIGHT;
	}
	return LCD_CTRL_WIDTH;
}


